package RNA::Auth::File;
#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================
# Authentication Module (for password file)
# 
# 認証用モジュール (パスワードファイル用)
#=============================================================================

use strict;
use CGI::Carp;
use vars qw($VERSION);

$VERSION = '0.1';

# $auth = new RNA::Auth::File(file=>'.rna-passwd');
sub new {
	my $class = shift;
	my $self = {};
	
	bless $self, $class;
	
	$self->init(@_);
	
	return $self;
}


sub init {
	my $self = shift;
	my %hash = @_;

	$self->_reset;

	my $file = $hash{file};
	$self->{_file} = $file;
}


# $self->_reset
sub _reset
{
	my $self = shift;

	$self->{_account} = {};
}


# パスワードファイルに反映
#
# $bool = $self->_write;
#
sub _write
{
	my $self = shift;
	my $file = $self->{_file};
	my $account = $self->{_account};

	open (FILE, ">$file") || croak "Can't open $file\n";
	flock(FILE, 2);
	while(my ($u, $p) = each %{$account}) {
		print FILE "$u:$p\n";
	}
	flock(FILE, 8);
	close(FILE);

	return 1;
}


# パスワードファイル読み込み
#
# $bool = $self->_read;
sub _read
{
	my $self = shift;
	my $file = $self->{_file};

	# reset
	$self->_reset;

	open (FILE, $file) || croak "Can't open $file\n";
	flock(FILE, 1);
	while(<FILE>) {
		s/\r\n/\n/g;
		chomp;
		my ($username, $passwd) = split(/:/);
		$self->{_account}->{$username} = $passwd;
	}
	flock(FILE, 8);
	close(FILE);

	return 1;
}


# $salt = $self->gen_salt;
sub gen_salt
{
	my $self = shift;
	my @table = ('a'..'z', 'A'..'Z', 0..9);

	return 
		$table[int(rand(scalar(@table) - 2))] 
		. $table[int(rand(scalar(@table) - 2))];
}

# $bool = $self->check($username, $passwd);
sub check
{
	my $self = shift;
	my ($username, $passwd) = @_;

	return 0 unless($username);

	# read passwd file
	$self->_read;

	my $account = $self->{_account};

	# 該当のユーザーが存在しない
	unless(exists $account->{$username}){
		$self->_reset;
		return 0;
	}

	my $valid_passwd = $account->{$username};

	# 真のパスワードが空で、入力パスワードが空でないとき
	if($valid_passwd eq '' && $passwd) {
		$self->_reset;
		return 0;
	}

	$self->_reset;
	return crypt($passwd, $valid_passwd) eq $valid_passwd;
}

# $bool = $self->add_user($username, $passwd);
sub add_user
{
	my $self = shift;
	my ($username, $passwd) = @_;

	return 0 unless($username);

	# read passwd file
	$self->_read;

	my $account = $self->{_account};

	# ユーザーが存在する
	if(exists $account->{$username}){
		return 0;
	}

	$account->{$username} = crypt($passwd, $self->gen_salt);

#print "Content-type: text/plain\n\n". join(':', %{$account});

	# write passwd file
	$self->_write;

	return 1;
}

# $bool = $self->update_user($username, $passwd);
sub update_user
{
	my $self = shift;
	my ($username, $passwd) = @_;

	return 0 unless($username);

	# read passwd file
	$self->_read;
	my $account = $self->{_account};

	# ユーザーが存在しない
	unless(exists $account->{$username}){
		return 0;
	}

	$account->{$username} = crypt($passwd, $self->gen_salt);

	# write passwd file
	$self->_write;
	return 1;
}

# $bool = $self->delete_user($username);
sub delete_user
{
	my $self = shift;
	my ($username) = @_;

	return 0 unless($username);

	# read passwd file
	$self->_read;
	my $account = $self->{_account};

	# ユーザーが存在しない
	unless(exists $account->{$username}){
		return 0;
	}

	delete $account->{$username};

	# write passwd file
	$self->_write;

	return 1;
}


#=============================================================================
1;
__END__

